<?php
################################################################################
##              -= YOU MAY NOT REMOVE OR CHANGE THIS NOTICE =-                 #
## --------------------------------------------------------------------------- #
##  ApPHP Tabs Pro (AJAX Enabled)  version 3.1.1                               #
##  Developed by:  ApPHP <info@apphp.com>                                      #
##  Last modified: 14.07.2011                                                  #
##  License:       GNU GPL v.3                                                 #
##  Site:          http://www.apphp.com/php-tabs/                              #
##  Copyright:     ApPHP Tabs (c) 2010 - 2011. All rights reserved.            #
##                                                                             #
################################################################################


class Tabs
{
    // PUBLIC               STATIC              PRIVATE
    // -------              -------             -------
    // constructor                              LoadFiles
    // AddTab                                   DisplayDropdown
    // AddTabAction                             DisplayRegular
    // Display                                  DisplayInnerContainer
    // DisablePossible                          DisplayTab
    // SetChildTabsType                         DisplayTabAsDropdown
    // GetLevel                                 DisplaySpan
    // SetHttpVars                              GetFormattedMicrotime
    // GetNumChildren                           ShowDebugInformation
    // IsSelected                               ToHex
    // GetId                                    CheckInput
    // GetCaption                               
    // SetCaption
    // SetWidth
    // SetHeight
    // GetWidth
    // GetHeight
    // SetAlign
    // GetAlign
    // GetSubmissionType
    // SetSubmissionType
    // SetDefaultTab
    // GetDefaultTabId
    // SetStyle
    // Debug
    // GetDebug
    // IsRefreshSelectedTabsAllowed
    // AllowRefreshSelectedTabs
    // UseAnchor
    // CreateAnchorAuto
    // GetParam
 
 
    //--- PRIVATE DATA MEMBERS --------------------------------------------------
    private $tabs;
    private $numOfSelectedTabs;
    private $numChildren=0;
    private $caption;
    private $id=1;
    private $align="left";
    private $width="auto";
    private $height="auto";
    private $path="";
    private $anchor="tabs";
    private $isDebug=false;
    private $submissionType="ajax";
    private $style="xp";
    private $defaultTabId="1";
    private $refreshSelectedTabsAllowed=true;
    private $useAnchor=false;
    private $createAnchorAuto=true;
    private $httpVars;
    private $childTabsType="tabs";
    private $maxLevel=0;
    private $code;
    private $codedTabs;
    private $crLt;
 
    // --- PUBLIC DATA MEMBERS --------------------------------------------------
    public $warningMessage;
 
    //--- CONSTANTS --------------------------------------------------
    const version="3.1.1";
 
    /**
     * Creates a new set of tabs 
     *      @param $id - this set's id
     *      @param $style - style's name (corresponds to subfolder of the same name within 'styles' folder)
     *      @param $caption - this set's name
     */
    public function __construct($id=1,$style="xp",$caption="")
    {
        if(is_numeric($id)&&is_integer($id)) $this->id=$id;
        $this->style=$style;
        $this->caption=$caption;
        $this->tabs=array();
        $this->id=$id;
        $this->httpVars=array();
        $this->codedTabs=array();
        $this->crLt = "\n";
        
        if(defined("TABS_DIR")) $this->path = TABS_DIR;
        else $this->path = "";
    }
 
    /**
     *  Should be used by user to add a new child tab to this set of tabs
     *  Actually it just calculates new tab's id and calls function AddTabAction
     *     @param $caption - text on the tab
     *     @param $file - file associated with this tab
     *     @param $icon - icon associated with this tab
     *     @param $tooltip - tab's tooltip
    */
    public function AddTab($caption,$file="",$icon="",$tooltip="")
    {
        $id=$this->GetId()."_".++$this->numChildren;
        return $this->AddTabAction($caption,$id,$this,$file,$icon,$tooltip);
    }
 
    /**
     *	Adds a new child tab to this set of tabs
     *     @param $caption - text on the tab
     *     @param $id - tab's id
     *     @param $directParent - tab which has this tab as a sub-tab (is set to $this when this tab is a first-level tab)
     *     @param $file - file associated with this tab
     *     @param $icon - icon associated with this tab
     *     @param $tooltip - tab's tooltip
    */
    public function AddTabAction($caption,$id,$directParent,$file="",$icon="",$tooltip="")
    {
        if(preg_match("/^[0-9|_]+$/",$id)==0) return;
        $level=substr_count($id,"_");
        if($level>$this->maxLevel)
        {
           $this->tabs[$level]=array();
           $this->maxLevel=$level;
        }
        $this->tabs[$level][$id]=new Tab($caption,$id,$this,$directParent,$file,$icon,$tooltip);
        return $this->tabs[$level][$id];
    }
 
    /**
     * Loads CSS and JS files
    */
    private function LoadFiles()
    {
        if(!file_exists($this->path."styles/".$this->style."/style.css")) $this->style="xp";
        echo  "<link href='".$this->path."styles/".$this->style."/style.css' rel='stylesheet' type='text/css' />";
        if($this->childTabsType=="dropdown") echo $this->crLt."<link href='".$this->path."styles/dropdown.css' rel='stylesheet' type='text/css' />";
        echo $this->crLt."<link href='".$this->path."styles/common.css' rel='stylesheet' type='text/css' />";
        echo $this->crLt."<script type='text/javascript' src='".$this->path."js/script.js'></script>";
        echo $this->crLt."<script type='text/javascript' src='".$this->path."js/jquery-1.4.2.min.js'></script>";
        echo $this->crLt."<!--[if IE]>";
        echo "<link href='".$this->path."styles/commonIE.css' rel='stylesheet' type='text/css' />";
        echo "<![endif]-->".$this->crLt;
        echo $this->crLt."<script type='text/javascript' src='".$this->path."js/jquery.dropdown.js'></script>";
    }
 
    /**
     *	Displays tabs when $childTabsType is set to 'dropdown'
     *      @param $preselected - selected tab's id
    */
    private function DisplayDropdown($preselected)
    {
        $output = $this->crLt."<ul class='tabsdropdown'>".$this->crLt;
        foreach($this->tabs[1] as $tab)
        { 
           $pos=strpos($preselected,$tab->GetId());
           if($pos ==0 && $pos!==false) $tab->Select();
           $output .= $this->DisplayTab($tab,false);
        }
        $output .= $this->crLt."</ul>";
        return $output;
    }
 
    /**
     *  Displays tabs when $childTabsType is set to 'tabs'
     *	    @param $preselected - selected tab's id
     *	    @param $level - selected tab's level
    */
    private function DisplayRegular($preselected,$level)
    {
        $output = "";
        if($this->submissionType=="ajax") $bound = $this->maxLevel;
        else $bound = $level;
        for($i=1;$i<=$bound;$i++)
        {
            if($level<$i) $output .= $this->crLt."<ul class='menu' style='display:none' id='menu".$this->id."-".$i."'>";
            else $output .= $this->crLt."<ul class='menu' id='menu".$this->id."-".$i."'>";
            foreach($this->tabs[$i] as $tab)
            {
                if($tab->GetDirectParent()->IsSelected())
                {
                    $pos=strpos($preselected,$tab->GetId());
                    if($pos ==0 && $pos!==false) $tab->Select();
                    $output .= $this->DisplayTab($tab,false);
                }
                else if($this->submissionType=="ajax")
                $output .= $this->DisplayTab($tab,true);
            }
            $output .= $this->crLt."</ul>";
        }
        return $output;
    }
 
    /**
     *	Displays tabs when $childTabsType is set to 'links'
     *      @param $preselected - selected tab's id
     *	    @param $level - selected tab's level
    */
    private function DisplaySubmenu($preselected,$level)
    {
        $output = "";
        if($this->submissionType=="ajax") $bound = $this->maxLevel;
        else $bound = $level;
        for($i=1;$i<=$bound;$i++)
        {
            if($i==2)
            {
                $output .= $this->crLt."<div class='innertabs' id='innertabs".$this->id."'";
                if($level==1) $output .= "style='display:none'";
                $output .= ">";
            }
            if($i==1) $menuclass='menu';
            else $menuclass='submenu';
            $drawLine = false;
            if($level<$i) $output .= $this->crLt."<ul class='".$menuclass."' style='display:none' id='menu".$this->id."-".$i."'>".$this->crLt;
            else $output .= $this->crLt."<ul class='".$menuclass."' id='menu".$this->id."-".$i."'>".$this->crLt;
            foreach($this->tabs[$i] as $tab)
            {
                if($tab->GetDirectParent()->IsSelected())
                {
                    $pos=strpos($preselected,$tab->GetId());
                    if($pos ==0 && $pos!==false) $tab->Select();
                    $output .= $this->DisplayTab($tab,false);
                    if($i>1 && $tab->GetNumber()!=$tab->GetDirectParent()->GetNumChildren())
                        $output .= "<li class='element' id='separator".$tab->GetId()."'><span class='vertseparator'><span class='inner'><span class='innerspan'>|</span></span></span></li>";
                    $drawLine = true;
                }else if($this->submissionType=="ajax"){
                    $output .= $this->DisplayTab($tab,true);
                    if($i>1 && $tab->GetNumber()!=$tab->GetDirectParent()->GetNumChildren())
                        $output .= "<li style='display:none' class='element' id='separator".$tab->GetId()."'><span class='vertseparator'><span class='inner'><span class='innerspan'>|</span></span></span></li>";
                }
            }
            $output .= $this->crLt."</ul>";
            if($i==1){
                if($this->submissionType!="ajax") $output .= $this->crLt."</form>";
                $output .= $this->crLt."<div id='container".$this->GetId()."' style='width:".$this->width.";height:".$this->height."' class='container'>";
            }else{
                $output .= $this->crLt."<table width='100%' height='1px' cellpadding='0' cellspacing='0' border='0' id='hrline".$this->id."-".$i."'";
                if(!$drawLine) $output .= "style='display:none'>";
                else $output .= "style=''>";
                $output .= $this->crLt."<tr class='separator1'><td class='separator'></td></tr>";
                $output .= $this->crLt."</table>";
            }
        }
        if($bound>=2) $output .= $this->crLt."</div>";
        
        return $output;
    }
 
    /**
     *	Displays container where content is displayed
     *      @param $preselected - selected tab's id
     *	    @param $level - selected tab's level
    */
    public function DisplayInnerContainer($level,$preselected)
    {
        $output = $this->crLt."<div id='innercontainer".$this->GetId()."' class='innercontainer'>".$this->crLt;
        $tab = isset($this->tabs[$level][$preselected]) ? $this->tabs[$level][$preselected] : null;
        if($tab != null && !$tab->HasCode()) $output .= $tab->ShowContent($this->isDebug);
        $output .= $this->crLt."</div>";
        $output .= $this->crLt."<div class='innercontainer'>".$this->crLt;
        foreach($this->codedTabs as $tabHard)
        {
            if($tab==$tabHard) $output .= $this->crLt."<div id='code".$tabHard->GetId()."'>".$tabHard->GetCode()."</div>";
            else if($this->submissionType=="ajax") $output .= $this->crLt."<div id='code".$tabHard->GetId()."' style='display:none'>".$tabHard->GetCode()."</div>";
        }
        $output .= $this->crLt."</div>";
        return $output;
    }
 
    /**
     *	Displays this set of tabs
    */
    public function Display()
    {
        if($this->useAnchor && $this->createAnchorAuto)
            echo "<a name=".$this->anchor."></a>";
        if($this->caption!="")
            echo "<span id='caption' class='caption'>".$this->caption."</span><br /><br />";
        $startTime = $this->GetFormattedMicrotime();
 
        $this->LoadFiles();
 
        $preselected=isset($_REQUEST['tabid'.$this->GetId()]) ? $_REQUEST['tabid'.$this->GetId()] : "";
 
        $level=substr_count($preselected, "_");
        if(!isset($this->tabs[$level][$preselected])||!$this->tabs[$level][$preselected]->IsEnabled())
            $currTab = $this;
        else $currTab = $this->tabs[$level][$preselected];
        while($currTab->GetNumChildren()>0)
            $currTab = $this->tabs[$currTab->GetLevel()+1][$currTab->GetId()."_".$currTab->GetDefaultTabId()];
        $preselected = $currTab->GetId();
        $level=substr_count($preselected, "_");
 
        if($this->submissionType!="ajax")
        {
            $params="";
            if($this->submissionType=="post")
            {
                foreach($this->httpVars as $key)
                {
                    if(self::GetParam($key, "get")!= "")
                        $params .= "&".$key."=".self::GetParam($key, "get");
                }
                if($params!="") $params[0]="?";
            }
            if($this->useAnchor) $action=$_SERVER["SCRIPT_NAME"].$params."#".$this->anchor;
            else $action=$_SERVER["SCRIPT_NAME"].$params;
            echo $this->crLt."<form class='frmtabs' name='frmtabs".$this->GetId()."' id='frmtabs".$this->GetId()."' action='".$action."' method='".$this->submissionType."'>".$this->crLt;
            if($this->submissionType=="get")
                foreach($this->httpVars as $key)
                    if(self::GetParam($key, "get")!= "")
                        echo $this->crLt."<input type='hidden' id='".$key."' name='".$key."' value='".self::GetParam($key, "get")."' />";
        }else{
            echo $this->crLt."<input type='hidden' id='path".$this->GetId()."' name='path' value='".$this->path."' />";
            echo $this->crLt."<input type='hidden' id='childStyle".$this->GetId()."' name='childStyle' value='".$this->childTabsType."' />";
            echo $this->crLt."<input type='hidden' id='maxlevel".$this->GetId()."' name='maxlevel' value='".$this->maxLevel."' />";
            echo $this->crLt."<input type='hidden' id='refresh".$this->GetId()."' value='".($this->refreshSelectedTabsAllowed ? "true" : "false") ."' />";
        }
        echo $this->crLt."<input type='hidden' id='tabid".$this->GetId()."' name='tabid".$this->GetId()."' value='".$preselected."' />";
        if($this->childTabsType=="dropdown")
        {
            echo $this->DisplayDropdown($preselected,$level);
            if($this->submissionType!="ajax") echo $this->crLt."</form>";
            echo $this->crLt."<div id='container".$this->GetId()."' style='width:".$this->width.";height:".$this->height."' class='container'>";
            echo $this->DisplayInnerContainer($level,$preselected);
            echo $this->crLt."</div>";
        }else if($this->childTabsType=="links"){
            echo $this->DisplaySubmenu($preselected,$level);
            echo $this->DisplayInnerContainer($level,$preselected);
            echo $this->crLt."</div>";
        }else{
            echo $this->DisplayRegular($preselected,$level);
            if($this->submissionType!="ajax") echo $this->crLt."</form>";
            echo $this->crLt."<div id='container".$this->GetId()."' style='width:".$this->width.";height:".$this->height."' class='container'>";
            echo $this->DisplayInnerContainer($level,$preselected);
            echo $this->crLt."</div>";
        }
 
        if($this->isDebug){
            echo $this->crLt;
            $this->ShowDebugInformation($startTime);
        }
        echo $this->crLt."<!-- END This script was generated by tabs.class.php v.".Tabs::version."(http://www.apphp.com/php-tabs/index.php) END -->".$this->crLt;
    }
 
    /**
     *	Gets a GET- or POST- parameter
     *      @param $key - parameter's key
     *	    @param $postbackMethod - postback method
    */
    public static function GetParam($key, $postbackMethod)
    {
        if(strtolower($postbackMethod)=="get")
            return isset($_GET[$key]) && self::CheckInput($_GET[$key]) ? $_GET[$key] : "";
        return isset($_POST[$key]) && self::CheckInput($_POST[$key]) ? $_POST[$key] : "";
    }
 
    /**
     *	Checks input string for suspicious code
     *      @param $input - input string
     *	    @param $level - security level
    */
    private static function CheckInput($input, $level = "medium")
    {
        if($input == "") return true;
        $error = 0;
        $bad_string = array("%20union%20", " /*", "*/ union /*", "+union+", "load_file", "outfile", "document.cookie", "onmouse", "<script", "<iframe", "<applet", "<meta", "<style", "<form", "<img", "<body", "<link", "_GLOBALS", "_REQUEST", "_GET", "_POST", "include_path", "prefix", "http://", "https://", "ftp://", "smb://" );
        foreach($bad_string as $string_value)
        {
            if(strstr($input, $string_value)) $error = 1;
        }
        if((preg_match("/<[^>]*script*\"?[^>]*>/i", $input)) ||
        (preg_match("/<[^>]*object*\"?[^>]*>/i", $input)) ||
        (preg_match("/<[^>]*iframe*\"?[^>]*>/i", $input)) ||
        (preg_match("/<[^>]*applet*\"?[^>]*>/i", $input)) ||
        (preg_match("/<[^>]*meta*\"?[^>]*>/i", $input)) ||
        (preg_match("/<[^>]*style*\"?[^>]*>/i", $input)) ||
        (preg_match("/<[^>]*form*\"?[^>]*>/i", $input)) ||
        (preg_match("/<[^>]*img*\"?[^>]*>/i", $input)) ||
        (preg_match("/<[^>]*onmouseover*\"?[^>]*>/i", $input)) ||
        (preg_match("/<[^>]*body*\"?[^>]*>/i", $input)) ||
        (preg_match("/\([^>]*\"?[^)]*\)/i", $input)) ||
        (preg_match("/ftp:\/\//i", $input)) ||
        (preg_match("/https:\/\//i", $input)) ||
        (preg_match("/http:\/\//i", $input)) )
        {
            $error = 1;
        }
        $ss = $_SERVER['HTTP_USER_AGENT'];
        if((preg_match("/libwww/i",$ss)) ||
        (preg_match("/^lwp/i",$ss))  ||
        (preg_match("/^Jigsaw/i",$ss)) ||
        (preg_match("/^Wget/i",$ss)) ||
        (preg_match("/^Indy\ Library/i",$ss)) )
        {
            $error = 1;
        }
        if($error){
            return false;
        }
        return true;
    }
 
    /**
     *	Displays a tab
     *      @param $tab - tab to display
     *      @param $hide - should the tab be hidden or not
    */
    private function DisplayTab($tab,$hide)
    {
        $output = "<li class='element' name='__tabsElement'";
        if($this->GetSubmissionType()=="ajax") $output .= " id='".$tab->GetId()."' ";
        if($hide) $output .= "style='display:none'";
        $output .= ">";
        $output .= $this->DisplaySpan($tab,"");
        if($this->childTabsType=="dropdown" && $tab->GetNumChildren()>0){
            $output .= "<ul>";
            for($i=1;$i<=$tab->GetNumChildren();$i++){
               $subtab = $this->tabs[$tab->GetLevel()+1][$tab->GetId()."_".$i];
               $output .= $this->DisplayTabAsDropdown($subtab);
            }
            $output .= "</ul>";
        }
        $output .= "</li>";        
        return $output;
    }
 
    /**
     *	Displays text on a tab
     *      @param $tab
    */
    private function DisplaySpan($tab)
    {
        if($tab->IsSelected()){
            if($this->refreshSelectedTabsAllowed) $style="sel";
            else $style="selpass";
        }else if($tab->IsEnabled()){
            $style="";
        }else{
            $style="dis";
        }
        $innerstyle='inner';
        
        $output = "<span";
 
        if($tab->GetTooltip()!="") $output .= " title='".$tab->GetTooltip()."'";
        if($tab->IsEnabled()){
            if($tab->IsOpenNewWindow()){
                $output .= " onclick='document.location.href=\"".$tab->GetFile()."\"'";
            }else{
                $pathToFile=substr($_SERVER["SCRIPT_NAME"],0,strrpos($_SERVER["SCRIPT_NAME"],"/")+1);
                if($this->GetSubmissionType()=="ajax"){
                    $currTab = $tab;
                    while($currTab->GetNumChildren()>0){
                        $currTab = $this->tabs[$currTab->GetLevel()+1][$currTab->GetId()."_".$currTab->GetDefaultTabId()];
                    }
                    $currId = $currTab->GetId();
                    if($currTab->HasCode()){
                        $output .= " onclick='__TabsPostBackAjax(this,\"".$currId."\",\"".$this->GetId()."\",\"code\")'";
                    }else{
                        if($tab->IsPicture()){
                            $output .= " onclick='__TabsPostBackAjax(this,\"".$currId."\", \"".$this->GetId()."\",\"file\",\"".$this->tabs[$currTab->GetLevel()][$currId]->GetFile()."\")'";
                        }else{
                            $output .= " onclick='__TabsPostBackAjax(this,\"".$currId."\", \"".$this->GetId()."\",\"file\",\"".$pathToFile.$this->tabs[$currTab->GetLevel()][$currId]->GetFile()."\")'";
                        }
                    }
                }else{
                    $output .= " id='".$tab->GetId()."' onclick='__TabsPostBack(this,\"".$this->GetId()."\")'";
                }
            }
        }
 
        $output .= " name='__tab' class='tab".$style."'>";
        $output .= "<table class='".$innerstyle."' align='center' border='0' cellpadding='0' cellspacing='0'><tr text-align='center'>";
        if($tab->GetIcon()!=""){
           $output .= "<td style='margin:0;padding-right:5px'><img style='vertical-align:middle;' src='".$tab->GetIcon()."'></td>";
        }
        if($this->childTabsType=="dropdown" && $tab->GetNumChildren()>0 && $tab->IsEnabled()) $output .= "<td style='margin:0;padding:0;'>";
        else $output .= "<td>";
        $output .= "<span class='innerspan'>".$tab->GetCaption()."</span>";
        $output .= "</td>";
        if($this->childTabsType=="dropdown" && $tab->GetNumChildren()>0 && $tab->IsEnabled()){
           $output .= "<td style='margin:0;padding-left:5px'>";
           $output .= "<img align='right' src='".$this->path."styles/indicator.gif' />";
           $output .= "</td>";
        }
        $output .= "</tr></table>";
        $output .= "</span>";
        
        return $output;
    }
 
    /**
     *	Displays a sub-tab when $childTabsType is set to 'dropdown'
     *      @param $tab - sub-tab to display
    */
    private function DisplayTabAsDropdown($tab)
    {
        $output = $this->crLt."<li";
        if($tab->IsEnabled()) $output .= " name='__tabsMenu' class='tabsMenu'";
        else $output .= " class='dropdis'";
        if($this->GetSubmissionType()=="ajax") $output .= " id='".$tab->GetId()."' ";
 
        if($this->GetSubmissionType()=="ajax")
        {
            $currTab = $tab;
            while($currTab->GetNumChildren()>0)
                $currTab = $this->tabs[$currTab->GetLevel()+1][$currTab->GetId()."_".$currTab->GetDefaultTabId()];
            $currId = $currTab->GetId();
        }
 
        if($tab->IsEnabled())
        {
            if($tab->IsOpenNewWindow())
            {
                $output .= " onclick='document.location.href=\"".$tab->GetFile()."\"'>";
            }else{
                $pathToFile=substr($_SERVER["SCRIPT_NAME"],0,strrpos($_SERVER["SCRIPT_NAME"],"/")+1);
                if($tab->GetNumChildren()==0)
                {
                    if($this->GetSubmissionType()=="ajax")
                    {
                        if($tab->HasCode())
                            $output .= " onclick='__TabsPostBackAjax(this,\"".$currId."\",\"".$this->GetId()."\",\"code\")'>";
                        else if($tab->IsPicture())
                            $output .= " onclick='__TabsPostBackAjax(this,\"".$currId."\", \"".$this->GetId()."\",\"file\",\"".$this->tabs[$currTab->GetLevel()][$currId]->GetFile()."\")'>";
                        else $output .= " onclick='__TabsPostBackAjax(this,\"".$currId."\", \"".$this->GetId()."\",\"file\",\"".$pathToFile.$this->tabs[$currTab->GetLevel()][$currId]->GetFile()."\")'>";
                    }
                    else $output .= " id='".$tab->GetId()."' onclick=\"__TabsPostBack(this,'".$this->GetId()."')\">";
                }
                else $output .= ">";
            }
        }
        else $output .= ">";
        $output .= "<table border='0' cellpadding='0' cellspacing='0'";
        if($tab->GetTooltip()!="") $output .= " title='".$tab->GetTooltip()."'";
        $output .= "><tr>";
        if($tab->GetIcon()!="")
        {
            $output .= "<td style='padding-right:5px'>";
            $output .= "<img style='vertical-align:middle' src='".$tab->GetIcon()."' />";
            $output .= "</td>";
        }
        $output .= "<td>".$tab->GetCaption()."&nbsp</td>";
        if($tab->GetNumChildren()>0 && $tab->IsEnabled())
        {
            $output .= "<td style='width:100%'>";
            $output .= "<img align='right' src='".$this->path."styles/extender.gif' />";
            $output .= "</td>";
        }
        $output .= "</tr></table>";
 
        if($tab->GetNumChildren()>0 && $tab->IsEnabled())
        {
            $output .= $this->crLt."<ul>";
            for($i=1;$i<=$tab->GetNumChildren();$i++)
            {
                $subtab = $this->tabs[$tab->GetLevel()+1][$tab->GetId()."_".$i];
                $output .= $this->DisplayTabAsDropdown($subtab);
            }
            $output .= $this->crLt."</ul>";
        }
        $output .= $this->crLt."</li>";
        
        return $output;
    }
 
    /**
     *   Adds a tab to list of tabs which are directly associated
     *   with HTML code and not with a file
     *      @param $tab
    */
    public function AddTabWithCode($tab)
    {
        if($tab->HasCode()) $this->codedTabs[count($this->codedTabs)]=$tab;
    }
 
    /**
     **   Checks if it's possible to disable a tab,
     *    i.e. makes sure it's not the only enabled tab on this level.
     *    Returns 'true' if it can be disabled and 'false' otherwise.
     *       @param $tab - tab to disable
    */
    public function DisablePossible($tab)
    {
        if(!is_a($tab,"Tab"))
        {
           if($this->isDebug) echo "<font color='#FF0000'>Error: 'Disable' function parameter is not a valid Tab object</font>";
           return;
        }
        $parentTab = $tab->GetDirectParent();
        if($parentTab->GetDefaultTabId()==$tab->GetNumber())
        {
            $possible = false;
            for($i=1;$i<=$parentTab->GetNumChildren();$i++)
            {
                $currTab = $this->tabs[$tab->GetLevel()][$parentTab->GetId().'_'.$i];
                if($currTab->IsEnabled() && $i!=$tab->GetNumber())
                {
                    $parentTab->SetDefaultTab($currTab);
                    return true;
                }
            }
            return false;
        }
        else return true;
    }
 
    /**
     *	Sets mode of displaying child tabs (dropdown|tabs|links)
     *       @param $childTabsType
    */
    public function SetChildTabsType($childTabsType)
    {
        $newChildTabsType = strtolower($childTabsType);
        if($newChildTabsType=="tabs" || $newChildTabsType=="links" || $newChildTabsType=="dropdown"){            
           $this->childTabsType=$newChildTabsType;
        }
    }
 
    /**
     *	Sets variables that were used to get access to the page (like: my_page.php?act=34&id=56 etc.)
     *       @param $vars
    */
    public function SetHttpVars($vars)
    {
        $this->httpVars = $vars;
    }
 
    /**
     *	Sets identifier for this set of tabs
     *       @param $id
    */
    public function SetId($id)
    {
        $this->id = $id;
    }
 
    /**
     *	Returns number of tabs on the first level
    */
    public function GetNumChildren()
    {
       return $this->numChildren;
    }
 
    /**
     *	Returns id of this set of tabs
    */
    public function GetId()
    {
       return $this->id;
    }
 
    /**
     *	Returns id of this set of tabs
    */
    public function Version()
    {
       return Tabs::version;
    }
 
    /**
     *	Returns caption of this set of tabs
    */
    public function GetCaption()
    {
        return $this->caption;
    }
 
    /**
     *	Sets caption of this set of tabs
     *     @param $caption - new caption
    */
    public function SetCaption($caption)
    {
        $this->caption=$caption;
    }
 
    /**
     *	Sets container's width
     *     @param $width - new width
    */
    public function SetWidth($width)
    {
        $width=strtolower($width);
        if($width=="auto"){
           $this->width="auto";
           return;
        }
        if(substr($width,-1)=="%"){
           $unit="%";
           $number=substr($width,0,strlen($width)-1);
        }else{
            $unit=substr($width,-2);
            if($unit=="px"||$unit=="pt"||$unit=="in"){
               $number=substr($width,0,strlen($width)-2);
            }else{
               $unit="px";
               $number=$width;
            }
        }
        if(preg_match("/^[0-9]+$/", $number)==0 || $number<0) return;
        if($unit=="%" && $number>100) return;
        $this->width=$number.$unit;
   }
 
    /**
     *	Returns container's width
    */
    public function GetWidth()
    {
        return $this->width;
    }
 
    /**
     *	Sets container's height
     *     @param $height - new height
    */
    public function SetHeight($height)
    {
        $height=strtolower($height);
        if($height=="auto"){
           $this->height="auto";
           return;
        }
        $unit=substr($height,-2);
        if($unit=="px"||$unit=="pt"||$unit=="in"){
           $number=substr($height,0,strlen($height)-2);
        }else{
           $unit="px";
           $number=$height;
        }
        if(preg_match("/^[0-9]+$/",$number)==0 || $number<0) return;
        $this->height=$number.$unit; 
    }
 
    /**
     *	Returns container's height
    */
    public function GetHeight()
    {
        return $this->height;
    }
 
    /**
     *	Sets mode of alignment within the container
     *     @param $align - left, center or right
    */
    public function SetAlign($align)
    {
        if(strtolower($align)=="center" || strtolower($align)=="left" || strtolower($align)=="right"){            
           $this->align=$align;
        }
    }
 
    /**
     *	Returns mode of alignment within the container
    */
    public function GetAlign()
    {
        return $this->align;
    }
 
    /**
     *	Returns form submission type
    */
    public function GetSubmissionType()
    {
        return $this->submissionType;
    }
 
    /**
     *	Sets form submission type
     *		@param $submission_type
    */
    public function SetSubmissionType($submission_type = "post")
    {
        if(strtolower($submission_type) == "get") $this->submissionType = "get";
        else if(strtolower($submission_type) == "ajax") $this->submissionType = "ajax";
        else $this->submissionType = "post";
    }
 
    /**
     *	Sets default tab
     *      @param $defaultTab - new default tab
    */
    public function SetDefaultTab($defaultTab)
    {
        if(!is_a($defaultTab,"Tab")){
            if($this->isDebug) echo "<font color='#FF0000'>Error: 'SetDefaultTab' function parameter is not a valid Tab object</font>";
            return;
        }
        $this->defaultTabId=$defaultTab->GetNumber();
    }
 
    /**
     *	Returns default tab's id
     *
    */
    public function GetDefaultTabId()
    {
        return $this->defaultTabId;
    }
 
    /**
     *	Sets style
     *     @param $style - new style
    */
    public function SetStyle($style)
    {
        if(file_exists($this->path."styles/".$style."/style.css")) $this->style=$style;
    }
 
    /**
     *	Sets debug mode
     *		@param $mode
    */
    public function Debug($mode = false)
    {
        if($mode === true || strtolower($mode) == "true") $this->isDebug = true;
    }
 
    /**
     *	Returns debug mode
    */
    public function GetDebug()
    {
        return $this->isDebug;
    }
 
    /**
     *	Returns if refreshing selected tabs is enabled
    */
    public function IsRefreshSelectedTabsAllowed()
    {
        return $this->refreshSelectedTabsAllowed;
    }
 
    /**
     *	Allows or prohibits to refresh selected tabs
     *		@param $allow - true or false
    */
    public function AllowRefreshSelectedTabs($allow = false)
    {
        if($allow === true || strtolower($allow) == "true") $this->refreshSelectedTabsAllowed = true;
        if($allow === false || strtolower($allow) == "false") $this->refreshSelectedTabsAllowed = false;
    }
 
    /**
     *	Use an anchor to navigate to tabs object after a tab is clicked
     *		@param $use - true or false
    */
    public function UseAnchor($use = false)
    {
        if($use === true || strtolower($use) == "true") $this->useAnchor = true;
        if($use === false || strtolower($use) == "false") $this->useAnchor = false;
    }
 
    /**
     *	Create the anchor automatically
     *		@param $create - true or false
    */
    public function CreateAnchorAuto($create = false)
    {
        if($create === true || strtolower($create) == "true") $this->createAnchorAuto = true;
        if($create === false || strtolower($create) == "false") $this->createAnchorAuto = false;
    }
 
    /**
     *	Gets formatted microtime
    */
    private function GetFormattedMicrotime()
    {
        list($usec, $sec) = explode(' ', microtime());
        return ((float)$usec + (float)$sec);
    }
 
    /**
     *	Shows debug information
     *      @param $startTime - time when the script started
    */
    private function ShowDebugInformation($startTime)
    {
        $endTime = $this->GetFormattedMicrotime();
        echo "<div style='margin: 10px auto; text-align:left; color:#000096;'>";
 
        echo "Debug Info: (Total running time: ".round((float)$endTime - (float)$startTime, 6)." sec.) <br />========<br />";
 
        if(count($this->tabs) == 0){    
            echo "ERRORS: <br />--------<br />";
            echo "1. No tabs defined!<br />";
            echo "<br />";
        }
        
        echo "TABS: <br />--------<br />";
        echo "<pre>";
        echo "<table style='color:#000096;'>";
        for($i=1; $i<=$this->maxLevel; $i++)
            foreach($this->tabs[$i] as $tab)
            {
                echo "<tr>";
                echo "<td>".$tab->GetId()."</td>";
                echo "<td>".$tab->GetCaption()."</td>";
                echo "</tr>";
            }
        echo "</table>";
        echo "</pre>";
        echo "<br />GET: <br />--------<br />";
        echo "<pre>";
 
        foreach(array_keys($_GET) as $key){
            if(self::GetParam($key, "get") != "") echo "[".$key."] => ".htmlspecialchars(self::GetParam($key, "get"))."<br/>";
        }
        echo "</pre><br />";
        echo "POST: <br />--------<br />";
        echo "<pre>";
        foreach(array_keys($_POST) as $key){
            if(self::GetParam($key, "post") != "") echo "[".$key."] => ".htmlspecialchars(self::GetParam($key, "post"))."<br/>";
        }
        echo "</pre><br />";
        if($this->submissionType!="ajax") echo $this->warningMessage;
        echo "</div>";
    }
 
    /**
     *	Converts a decimal number to two-digit hexadecimal
     *		@param $number - number to be converted
    */
    private static function ToHex($number)
    {
       if(strlen(dechex($number)) > 1) return dechex($number);
       else return "0".dechex($number);
    }
 
    /**
     *	Function always returns 0 and exists just for recursive purposes
    */
    public function GetLevel()
    {
       return 0;
    }
 
    /**
     *	Function always returns true and exists just for recursive purposes
     */
    public function IsSelected()
    {
       return true;
    }
 
    /**
     *	Function always returns true and exists just for recursive purposes
    */
    public function IsEnabled()
    {
        return true;
    }
}
 
/**
 *	class Tab
 *      represents a separate tab
 *      last date modified: 01.05.2011
 */
class Tab
{
    // PUBLIC              PRIVATE 
    // -------             -------
    // constructor         IsHTML
    // AddTab              IsPHP
    // ShowContent         IsText
    // SetInnerHTML
    // HasCode
    // GetCode
    // IsSelected
    // Select
    // Deselect
    // IsEnabled
    // Enable
    // Disable
    // GetCaption
    // GetLevel
    // GetTooltip
    // SetTooltip
    // GetDirectParent
    // GetId
    // GetNumber
    // GetFile
    // GetIcon
    // SetIcon
    // OpenNewWindow
    // IsOpenNewWindow
    // GetNumChildren
    // SetDefaultTab
    // GetDefaultTabId
    // IsPicture
 
 
    //--- PRIVATE DATA MEMBERS --------------------------------------------------
    private $caption;
    private $enabled=true;
    private $selected=false;
    private $id;
    private $parent;
    private $directParent;
    private $defaultTabId="1";
    private $numChildren=0;
    private $code="";
    private $openNewWindow;
    private $level;
    private $tooltip;
    private $icon;
 

    /**
     *	Creates a new tab
     *     @param $caption - tab's caption
     *     @param $id - tab's id
     *     @param $parent - Tabs object which contains this tab
     *     @param $directParent - tab which contains this tab as a sub-tab
     *     @param $file - name of the file associated with this tab
     *     @param $icon - icon associated with this tab
     *     @param $tooltip - tab's tooltip
    */
    public function __construct($caption,$id,$parent,$directParent,$file="",$icon="",$tooltip="")
    {
        if(preg_match("/^[0-9|_]/",$id)==0) $id=0;
        $this->file=$file;
        $this->id=$id;
        $this->caption=str_replace(' ','&#160;',$caption);
        $this->icon=$icon;
        $this->parent=$parent;
        $this->directParent=$directParent;
        $this->level=substr_count($id,"_");
        $this->number=substr(strrchr($id, "_"),1);
        $this->SetTooltip($tooltip);
    }
 
    /**
     *	Adds a new child tab to this tab
     *      @param $caption - text on the tab
     *      @param $file - file associated with this tab
     *      @param $icon - icon associated with this tab
    */
    public function AddTab($caption,$file="",$icon="")
    {
        if(!is_a($this->parent,"Tabs")){
           echo "<font color='#FF0000'>Error: tab ".$this->caption." has no valid parent object</font>";
           return;
        }
        $id=$this->GetId()."_".++$this->numChildren;
        return $this->parent->AddTabAction($caption,$id,$this,$file,$icon);
    }
 
    /**
     *	Shows contents of the file which is associated with this tab
    */
    public function ShowContent()
    {
        $output = "";
        if($pos=strrpos($this->file,"?"))
        {
            $get_parameters=substr($this->file,$pos+1);
            $this->file=substr($this->file,0,$pos);
            $get_parameters_array = explode("&", $get_parameters);
            foreach($get_parameters_array as $get_parameter)
            {
                $key_value=explode("=",$get_parameter);
                $_GET[$key_value[0]]=$key_value[1];
            }
        }
        $output .= "<br />";
        if($this->IsPicture()){
            $output .= "<img src='".$this->file."' />";
        }else{
            if(!file_exists($this->file)){
                $this->parent->warningMessage = "File '".$this->file."' not found";
            }else if($this->IsPHP()){
                flush(); 
                ob_start(); 
                include_once($this->file);
                $contents = ob_get_clean(); 
                $output .= $contents;
            }else if($this->IsHTML()){
                $str = file_get_contents($this->file);
                if(preg_match("/<head.*?>(.+?)<\/head>/si",$str,$head) != 0){
                    if(preg_match_all("/<script.*?>(.*?)<\/script>/si",$head[1],$scripts) != 0)
                        foreach($scripts[0] as $script){ 
                            $output .= $script;
                        }
                    if(preg_match_all("/<style.*?>(.*?)<\/style>/si",$head[1],$styles) != 0){
                        foreach($styles[0] as $style){
                            $output .= $style;
                        }
                    }
                }
                if(preg_match("/<body.*?>(.+?)<\/body>/si",$str,$body) != 0){
                    $output .= print_r($body[1], true);
                }else{
                    $output .= print_r($str, true);
                }
            }else if($this->IsText()){
                $output .= htmlspecialchars(file_get_contents($this->file), ENT_QUOTES);
            }
        }
        return $output; 
    }
 
    /**
     *	Associates the tab with HTML code snippet
     *      @param $code
    */
    public function SetInnerHTML($code)
    {
        $this->code = $code;
        if($code!="") $this->parent->AddTabWithCode($this);
    }
 
    /**
     *	Checks if the tab is associated with HTML code snippet
     *    (not with a file as it is by default)
    */
    public function HasCode()
    {
       return $this->code!="";
    }
 
    /**
     *	Returns HTML code snippet which is associated with the tab
    */
    public function GetCode()
    {
       return $this->code;
    }
 
    /**
     *	Checks if the tab is selected
    */
    public function IsSelected()
    {
       return $this->selected;
    }
 
    /**
     *	Selects the tab
    */
    public function Select()
    {
        if(!$this->IsEnabled()) $this->Enable();
        $this->selected=true;
    }
 
    /**
     *	Deselects the tab
    */
    public function Deselect()
    {
        $this->selected=false;
    }
 
    /**
     *	Checks if the tab is enabled
    */
    public function IsEnabled()
    {
        if(!$this->GetDirectParent()->IsEnabled()) return false;
        return $this->enabled;
    }
 
    /**
     *	Enables the tab
    */
    public function Enable()
    {
        $this->enabled=true;
    }
 
    /**
     *	Disables the tab
    */
    public function Disable()
    {
        $possible = $this->parent->DisablePossible($this);
        if($possible) $this->enabled=false;
    }
 
    /**
     *	Returns the tab's caption
    */
    public function GetCaption()
    {
        return $this->caption;
    }
 
    /**
     *	Returns the tab's level
    */
    public function GetLevel()
    {
        return $this->level;
    }

    /**
     *	Returns the tab's tooltip
    */
    public function GetTooltip()
    {
        return $this->tooltip;
    }

    /**
     *	Sets tab's tooltip
     *       @param $tooltip
    */
    public function SetTooltip($tooltip)
    {
        $this->tooltip= htmlspecialchars($tooltip, ENT_QUOTES);
    }
 
    /**
     *	Returns tab which has this tab as a sub-tab
    */
    public function GetDirectParent()
    {
        return $this->directParent;
    }
 
    /**
     *	Returns the tab's id
    */
    public function GetId()
    {
        return $this->id;
    }
 
    /**
     *	Returns the tab's number
     *
    */
    public function GetNumber()
    {
        return $this->number;
    }
  
    /**
     *	Returns name of the file which is associated with this tab
    */
    public function GetFile()
    {
        return $this->file;
    }

    /**
     *	Sets icon which is associated with this tab
    */
    public function SetIcon($icon)
    {
        $this->icon=$icon;
    }

    /**
     *	Returns name of the file which contains icon associated with this tab
    */
    public function GetIcon()
    {
        return $this->icon;
    }

    /**
     *	Sets if this tab must be opened in a new window
     *     @param $open
    */
    public function OpenNewWindow($open = false)
    {
        if($open === true || strtolower($open) == "true") $this->openNewWindow = true;
    }

    /**
     *	Returns if this tab must be opened in a new window
    */
    public function IsOpenNewWindow()
    {
        return $this->openNewWindow;
    }
  
    /**
     *	Returns amount of subtabs associated with this tab
    */
    public function GetNumChildren()
    {
        return $this->numChildren;
    }
  
    /**
     *	Sets default tab
     *    @param $defaultTab - new default tab
    */
    public function SetDefaultTab($defaultTab)
    {
        if(!is_a($defaultTab,"Tab")){
            if($this->parent->GetDebug()) echo "<font color='#FF0000'>Error: 'SetDefaultTab' function parameter is not a valid Tab object</font>";
            return;
        }
        if($defaultTab->IsEnabled()){
            $this->defaultTabId=$defaultTab->GetNumber();
        }
    }
  
    /**
     *	Returns default tab's id
    */
    public function GetDefaultTabId()
    {
        return $this->defaultTabId;
    }
 
    /**
     *	Checks if file associated with this tab is a graphic file
    */
    public function IsPicture()
    {
        $extension=strtolower(substr(strrchr($this->file,"."),1));
        if($extension=="jpg" || $extension=="gif" || $extension=="bmp" || $extension=="tif" || $extension=="png" || $extension=="jpeg"){
            return true;
        }
        else return false;
    }
  
    /**
     *	Checks if file associated with this tab is a hypertext file
    */
    private function IsHTML()
    {
        $extension=strtolower(substr(strrchr($this->file,"."),1));
        if($extension=="htm" || $extension=="xml" || $extension=="html") return true;
        else return false;
    }
  
    /**
     *	Checks if file associated with this tab is a PHP file
    */
    private function IsPHP()
    {
        $extension=strtolower(substr(strrchr($this->file,"."),1));
        if($extension=="php") return true;
        else return false;
    }
  
    /**
     *	Checks if file associated with this tab is a PHP file
    */
    private function IsText()
    {
        $extension=strtolower(substr(strrchr($this->file,"."),1));
        if($extension=="txt") return true;
        else return false;
    }
}

?>